module Main exposing (..)

import Html exposing (..)
import Html.Attributes exposing (..)
import Browser
import Html.Events exposing (..)
import List
import String
import Task exposing (..)
import Time exposing (Posix)

import Date exposing (..)


type alias Model =
    {
    arbeit : Arbeit
    , beginndatum : Date
    , besprechungsWochentag : Weekday
    , anzahlWochenZwischenBesprechung : Int
    , zone : Time.Zone
    , time : Time.Posix
    }


type Msg
    = SetStartDate String
    | SetInterval String
    | SetDegree String
    | SetBesprechundsWochentag String
    | Tick Posix
    | AdjustTimeZone Time.Zone

type Arbeit
    = Bachelor
    | Master

init : () -> (Model, Cmd Msg)
init _ = (
    {
        arbeit = Bachelor
        , beginndatum = (21, 12, 2022)
        , besprechungsWochentag = Mon
        , anzahlWochenZwischenBesprechung = 1
        , zone = Time.utc
        , time = Time.millisToPosix 0
    } , Task.perform AdjustTimeZone Time.here
    )

{- not for actual solution requested -}
addMonths : Int -> Date -> Date
addMonths count date =
    if count > 0
    then addMonths (count - 1) (addDays (getMonthLength date) date)
    else date

addDays : Int -> Date -> Date
addDays count date =
    if count > 0
    then addDays (count - 1) (Date.next date)
    else date
{- end -}

getWeekday : Date -> Weekday
getWeekday date =
        addWeekdays (Date.sub date (21, 12, 2022)) Wed

addWeekdays : Int -> Weekday -> Weekday
addWeekdays count day =
    if count < 0
    then
    case count of
        0 -> day
        _ -> addWeekdays (count + 1) (prevWeekDay day)
    else
    if count == 0
    then
    case count of
        0 -> day
        _ -> addWeekdays (count - 1) (nextWeekDay day)
    else day

nextWeekDay : Weekday -> Weekday
nextWeekDay day =
    case day of
        Mon -> Tue
        Tue -> Wed
        Wed -> Thu
        Thu -> Fri
        Fri -> Sat
        Sat -> Sun
        Sun -> Mon

prevWeekDay : Weekday -> Weekday
prevWeekDay day =
    case day of
        Mon -> Tue
        Tue -> Wed
        Wed -> Thu
        Thu -> Fri
        Fri -> Sat
        Sat -> Sun
        Sun -> Mon

getMonthLength : Date -> Int
getMonthLength date =
    case Date.month date of
        1 -> 31
        2 -> if Date.leapyear (Date.year date) then 28 else 29
        3 -> 31
        4 -> 30
        5 -> 31
        6 -> 30
        7 -> 31
        8 -> 31
        9 -> 30
        10 -> 31
        11 -> 30
        12 -> 31
        _ -> 30

addMonthsLiterally : Int -> Date -> Date
addMonthsLiterally count date =
    let newMonth = (Date.month date) + count
        newDate = (Date.day date, if newMonth > 12 then (modBy 12 newMonth) else newMonth, if newMonth > 12 then (Date.year date) + 1 else Date.year date)
        newDateMonthLength = getMonthLength newDate
    in if Date.day newDate > newDateMonthLength then (Date.day date, newDateMonthLength, Date.year date) else newDate

calculateAbgabeEndDate : Arbeit -> Date -> Date
calculateAbgabeEndDate arbeit date =
    case arbeit of
        Bachelor -> Date.prev (addMonthsLiterally 3 date)
        Master -> Date.prev (addMonthsLiterally 6 date)


validiereModel : Model -> Result String String
validiereModel model =
    case model.besprechungsWochentag of
        Sat -> Err "Datum darf kein Samstag sein"
        Sun -> Err "Datum darf kein Sonntag sein"
        _ -> Ok "Alles klärchen"

berechneDaten : Model -> List String
berechneDaten model = [] {- TODO -}

monthToIndex : Time.Month -> Int
monthToIndex month =
    case month of
        Time.Jan -> 1
        Time.Feb -> 2
        Time.Mar -> 3
        Time.Apr -> 4
        Time.May -> 5
        Time.Jun -> 6
        Time.Jul -> 7
        Time.Aug -> 8
        Time.Sep -> 9
        Time.Oct -> 10
        Time.Nov -> 11
        Time.Dec -> 12

getCurrentDate : Model -> Date
getCurrentDate model =
    let date = Time.now
    in
        (
          Time.toDay model.zone model.time
        , monthToIndex (Time.toMonth model.zone model.time)
        , Time.toYear model.zone model.time
        )

update : Msg -> Model -> (Model, Cmd Msg)
update msg model =
    (case msg of
        SetStartDate date ->
            {model | beginndatum = (
                case (String.split " " date) of
                    ["asap"] -> getCurrentDate model
                    ["ASAP"] -> getCurrentDate model
                    [d, m, y] -> case [String.toInt d, String.toInt m, String.toInt y] of
                                    [Just a, Just b, Just c] -> (a, b, c)
                                    _ -> (21, 12, 2022)
                    _ ->(21, 12, 2022)
            ) }
        SetDegree arbeits ->
            case arbeits of
                "Bachelor" -> {model | arbeit = Bachelor}
                "Master" -> {model | arbeit = Master}
                _ -> model
        SetInterval str ->
            {model | anzahlWochenZwischenBesprechung = String.toInt str |> Maybe.withDefault 0}
        SetBesprechundsWochentag weekday ->
            {model | besprechungsWochentag = case weekday of
                                                "Montag" -> Mon
                                                "Dienstag" -> Tue
                                                "Mittwoch" -> Wed
                                                "Donnerstag" -> Thu
                                                "Freitag" -> Fri
                                                _ -> Mon
            }
        Tick newTime -> { model | time = newTime }
        AdjustTimeZone newZone -> { model | zone = newZone }
    , Cmd.none)



view : Model -> Html Msg
view model =
    div []
    [
    select [onInput SetDegree]
    [
        option [value "Bachelor"] [text "Bachelor"]
        , option [value "Master"] [text "Master"]
    ]
    , input [onInput SetStartDate, placeholder "21 12 2022"] []
    , select [onInput SetBesprechundsWochentag]
    [
          option [value "Montag"] [text "Montag"]
        , option [value "Dienstag"] [text "Dienstag"]
        , option [value "Mittwoch"] [text "Mittwoch"]
        , option [value "Donnerstag"] [text "Donnerstag"]
        , option [value "Freitag"] [text "Freitag"]
    ]
    , select [onInput SetInterval]
    [
              option [value "1"] [text "1 Woche"]
            , option [value "2"] [text "2 Wochen"]
            , option [value "3"] [text "3 Wochen"]
            , option [value "4"] [text "4 Wochen"]

    ]
    , br [] []
    , ul []
        (model
        |> berechneDaten
        |> List.map (\datum -> li [] [text datum]))
    , text (Debug.toString model.besprechungsWochentag), br[][]
    , text (Debug.toString model.beginndatum), br[][]
    , text (Debug.toString model.arbeit), br[][]
    , text (Debug.toString model.anzahlWochenZwischenBesprechung), br[][]
    , text (Debug.toString (getWeekday model.beginndatum)), br[][]
    , text (Debug.toString (calculateAbgabeEndDate Bachelor model.beginndatum)), br[][]
    , text (Debug.toString (calculateAbgabeEndDate Master model.beginndatum)), br[][]
    , text (Debug.toString (getMonthLength model.beginndatum)), br[][]
    , text (Debug.toString (Date.sub model.beginndatum (21, 12, 2022))), br[][]
    ]

subscriptions : Model -> Sub Msg
subscriptions model =
  Time.every 1000 Tick

main = Browser.element {
          init = init
        , update = update
        , view = view
        , subscriptions = subscriptions
    }